@@ -4,6 +4,7 @@ use crate::f16::f16;
 use crate::tensor::litteral::*;
 use crate::tensor::Tensor;
 use crate::TVec;
+use num_complex::Complex;
 use std::hash::Hash;
 use std::{fmt, ops};
 
@@ -97,6 +98,12 @@ pub enum DatumType {
     String,
     QI8(QParams),
     QU8(QParams),
+    ComplexI16,
+    ComplexI32,
+    ComplexI64,
+    ComplexF16,
+    ComplexF32,
+    ComplexF64,
 }
 
 impl DatumType {
@@ -105,6 +112,10 @@ impl DatumType {
         if *self == String || *self == TDim || *self == Blob || *self == Bool || self.is_quantized()
         {
             tvec!(*self)
+        } else if self.is_complex_float() {
+            [ComplexF16, ComplexF32, ComplexF64].iter().filter(|s| s.size_of() >= self.size_of()).copied().collect()
+        } else if self.is_complex_signed() {
+            [ComplexI16, ComplexI32, ComplexI64].iter().filter(|s| s.size_of() >= self.size_of()).copied().collect()
         } else if self.is_float() {
             [F16, F32, F64].iter().filter(|s| s.size_of() >= self.size_of()).copied().collect()
         } else if self.is_signed() {
@@ -147,28 +158,32 @@ impl DatumType {
     }
 
     pub fn is_unsigned(&self) -> bool {
-        match self.unquantized() {
-            DatumType::U8 | DatumType::U16 | DatumType::U32 | DatumType::U64 => true,
-            _ => false,
-        }
+        matches!(self.unquantized(), DatumType::U8 | DatumType::U16 | DatumType::U32 | DatumType::U64)
     }
 
     pub fn is_signed(&self) -> bool {
-        match self.unquantized() {
-            DatumType::I8 | DatumType::I16 | DatumType::I32 | DatumType::I64 => true,
-            _ => false,
-        }
+        matches!(self.unquantized(), DatumType::I8 | DatumType::I16 | DatumType::I32 | DatumType::I64)
     }
 
     pub fn is_float(&self) -> bool {
-        match self {
-            DatumType::F16 | DatumType::F32 | DatumType::F64 => true,
-            _ => false,
-        }
+        matches!(self, DatumType::F16 | DatumType::F32 | DatumType::F64)
+    }
+
+    pub fn is_complex(&self) -> bool {
+        self.is_complex_float() || self.is_complex_signed()
+    }
+
+    pub fn is_complex_float(&self) -> bool {
+        matches!(self, DatumType::ComplexF16 | DatumType::ComplexF32 | DatumType::ComplexF64)
+    }
+
+    pub fn is_complex_signed(&self) -> bool {
+        matches!(self, DatumType::ComplexI16 | DatumType::ComplexI32 | DatumType::ComplexI64)
     }
 
     pub fn is_copy(&self) -> bool {
         *self == DatumType::Bool || self.is_unsigned() || self.is_signed() || self.is_float()
+            || self.is_complex()
     }
 
     pub fn is_quantized(&self) -> bool {
@@ -293,6 +308,12 @@ impl std::str::FromStr for DatumType {
             "Blob" | "blob" => Ok(DatumType::Blob),
             "String" | "string" => Ok(DatumType::String),
             "TDim" | "tdim" => Ok(DatumType::TDim),
+            "ComplexI16" | "complexi16" => Ok(DatumType::ComplexI16),
+            "ComplexI32" | "complexi32"  => Ok(DatumType::ComplexI32),
+            "ComplexI64" | "complexi64"  => Ok(DatumType::ComplexI64),
+            "ComplexF16" | "complexf16"  => Ok(DatumType::ComplexF16),
+            "ComplexF32" | "complexf32"  => Ok(DatumType::ComplexF32),
+            "ComplexF64" | "complexf64"  => Ok(DatumType::ComplexF64),
             _ => anyhow::bail!("Unknown type {}", s),
         }
     }
@@ -382,6 +403,12 @@ datum!(u64, U64);
 datum!(TDim, TDim);
 datum!(String, String);
 datum!(Blob, Blob);
+datum!(Complex<i16>, ComplexI16);
+datum!(Complex<i32>, ComplexI32);
+datum!(Complex<i64>, ComplexI64);
+datum!(Complex<f16>, ComplexF16);
+datum!(Complex<f32>, ComplexF32);
+datum!(Complex<f64>, ComplexF64);
 
 #[cfg(test)]
 mod tests {
