@@ -5,6 +5,7 @@ use crate::f16::f16;
 use crate::TVec;
 use itertools::Itertools;
 use ndarray::prelude::*;
+use num_complex::Complex;
 #[cfg(feature = "serialize")]
 use serde::ser::{Serialize, Serializer};
 use std::alloc;
@@ -23,6 +24,7 @@ pub struct Tensor {
     dt: DatumType,
     shape: TVec<usize>,
     strides: TVec<isize>,
+    len: usize,
     layout: alloc::Layout,
     data: *mut u8,
 }
@@ -55,6 +57,12 @@ impl Hash for Tensor {
                 Blob => self.as_slice_unchecked::<crate::datum::Blob>().hash(state),
                 QI8(_) => self.as_slice_unchecked::<i8>().hash(state),
                 QU8(_) => self.as_slice_unchecked::<u8>().hash(state),
+                ComplexI16 => self.as_slice_unchecked::<Complex<i16>>().hash(state),
+                ComplexI32 => self.as_slice_unchecked::<Complex<i32>>().hash(state),
+                ComplexI64 => self.as_slice_unchecked::<Complex<i64>>().hash(state),
+                ComplexF16 => self.as_slice_unchecked::<Complex<i16>>().hash(state),
+                ComplexF32 => self.as_slice_unchecked::<Complex<i32>>().hash(state),
+                ComplexF64 => self.as_slice_unchecked::<Complex<i64>>().hash(state),
             }
         }
     }
@@ -146,7 +154,7 @@ impl Tensor {
             assert!(!ptr.is_null());
             ptr
         } as *mut u8;
-        let mut tensor = Tensor { strides: tvec!(), layout, dt, shape: shape.into(), data };
+        let mut tensor = Tensor { strides: tvec!(), layout, dt, shape: shape.into(), data, len: 0 };
         #[cfg(debug_assertions)]
         {
             if dt == DatumType::F32 {
@@ -155,7 +163,7 @@ impl Tensor {
                 tensor.as_slice_mut_unchecked::<i32>().iter_mut().for_each(|f| *f = std::i32::MIN)
             }
         }
-        tensor.update_strides();
+        tensor.update_strides_and_len();
         Ok(tensor)
     }
 
@@ -188,6 +196,12 @@ impl Tensor {
                 DatumType::String => String::stack_tensors(axis, &tensors),
                 DatumType::QI8(_) => i8::stack_tensors(axis, &tensors),
                 DatumType::QU8(_) => i8::stack_tensors(axis, &tensors),
+                DatumType::ComplexI16 => Complex::<i16>::stack_tensors(axis, &tensors),
+                DatumType::ComplexI32 => Complex::<i32>::stack_tensors(axis, &tensors),
+                DatumType::ComplexI64 => Complex::<i64>::stack_tensors(axis, &tensors),
+                DatumType::ComplexF16 => Complex::<i16>::stack_tensors(axis, &tensors),
+                DatumType::ComplexF32 => Complex::<i32>::stack_tensors(axis, &tensors),
+                DatumType::ComplexF64 => Complex::<i64>::stack_tensors(axis, &tensors),
             }
         }?;
         tensor.dt = dt;
@@ -237,7 +251,28 @@ impl Tensor {
         }
     }
 
-    /// Create an tensor from raw data.
+    /// Create a tensor with a given shape and a slice of elements. 
+    /// The data is copied and aligned to size of T. 
+    pub fn from_shape<T: Datum + Copy>(shape: &[usize], data: &[T]) -> anyhow::Result<Tensor> {
+        let dt = T::datum_type();
+        Self::from_shape_align(shape, data, dt.alignment())
+    }
+
+    /// Create a tensor with a given shape and a slice of elements. 
+    /// The data is copied and aligned to given alignment. 
+    pub fn from_shape_align<T: Datum + Copy>(shape: &[usize], data: &[T], align: usize) -> anyhow::Result<Tensor> {
+        anyhow::ensure!(data.len() == shape.iter().product::<usize>(), "Shape product must be equal to data length");
+        unsafe {
+            let bytes = std::slice::from_raw_parts(
+                data.as_ptr() as *const u8,
+                data.len() * T::datum_type().size_of(),
+            );
+            let dt = T::datum_type();
+            Self::from_raw_dt_align(dt, shape, bytes, align)
+        }
+    }
+
+    /// Create a tensor from raw data.
     ///
     /// It copies the data, aligning it to the size of T.
     pub unsafe fn from_raw<T: Datum>(shape: &[usize], content: &[u8]) -> anyhow::Result<Tensor> {
@@ -295,8 +330,9 @@ impl Tensor {
     }
 
     /// Get the number of valeus in the tensor.
+    #[inline]
     pub fn len(&self) -> usize {
-        self.shape.iter().cloned().product::<usize>()
+        self.len
     }
 
     /// Get the shape of the tensor.
@@ -305,9 +341,14 @@ impl Tensor {
         &self.strides
     }
 
-    fn update_strides(&mut self) {
+    fn update_strides_and_len(&mut self) {
         self.strides.clear();
         compute_natural_stride_to(&mut self.strides, &self.shape);
+        self.len = if self.rank() == 0 {
+            1
+        } else {
+            unsafe { *self.strides.get_unchecked(0) as usize * self.shape.get_unchecked(0) }
+        }
     }
 
     /// Force the tensor shape, no consistency check.
@@ -315,7 +356,7 @@ impl Tensor {
         if shape != &*self.shape {
             self.shape.clear();
             self.shape.extend_from_slice(shape);
-            self.update_strides();
+            self.update_strides_and_len();
         }
     }
 
@@ -341,6 +382,35 @@ impl Tensor {
         }
     }
 
+    pub fn move_axis(self, from: usize, to: usize) -> anyhow::Result<Tensor> {
+        let mut permutation: Vec<usize> = (0..self.rank()).collect();
+        permutation.remove(from);
+        permutation.insert(to, from);
+        self.permute_axes(&permutation)
+    }
+
+    pub fn collapse_axis_with_next(mut self, axis: usize) -> Tensor {
+        let removed = self.shape.remove(axis + 1);
+        self.shape[axis] *= removed;
+        self.update_strides_and_len();
+        self
+    }
+
+    pub fn split_axis(mut self, axis: usize, outer_dim: usize) -> anyhow::Result<Tensor> {
+        if self.shape[axis] % outer_dim != 0 {
+            anyhow::bail!(
+                "Invalid axis split, shape is {:?}, axis split at {}, outer {}",
+                self.shape,
+                axis,
+                outer_dim
+            );
+        }
+        self.shape.insert(axis + 1, self.shape[axis] / outer_dim);
+        self.shape[axis] = outer_dim;
+        self.update_strides_and_len();
+        Ok(self)
+    }
+
     /// Reshape the tensor to `shape`.
     pub fn into_shape(mut self, shape: &[usize]) -> anyhow::Result<Tensor> {
         self.set_shape(shape)?;
@@ -361,7 +431,7 @@ impl Tensor {
 
     pub fn broadcast_into_rank(mut self, rank: usize) -> anyhow::Result<Tensor> {
         self.broadcast_to_rank(rank)?;
-        self.update_strides();
+        self.update_strides_and_len();
         Ok(self)
     }
 
@@ -372,7 +442,7 @@ impl Tensor {
         while self.shape.len() < rank {
             self.shape.insert(0, 1)
         }
-        self.update_strides();
+        self.update_strides_and_len();
         Ok(())
     }
 
@@ -517,11 +587,13 @@ impl Tensor {
     }
 
     /// Get the datum type of the tensor.
+    #[inline]
     pub fn datum_type(&self) -> DatumType {
         self.dt
     }
 
     /// Set the datum type of the tensor.
+    #[inline]
     pub unsafe fn set_datum_type(&mut self, dt: DatumType) {
         self.dt = dt
     }
@@ -997,8 +1069,9 @@ impl Tensor {
             let shape = it.shape().into();
             let vec = it.into_raw_vec().into_boxed_slice();
             let data = Box::into_raw(vec) as *mut u8;
-            let mut t = Tensor { dt: T::datum_type(), shape, layout, data, strides: tvec!() };
-            t.update_strides();
+            let mut t =
+                Tensor { dt: T::datum_type(), shape, layout, data, strides: tvec!(), len: 0 };
+            t.update_strides_and_len();
             return t;
         }
         unsafe {
